
// alphabet is newly inserted and all methods have to be checked against this
// regexp import Automaton::Automaton(Regexp r)
// multiple start states ?
// coordinate-assign (+ make sure l is not exceeded)
// set<int> zum sammeln aller finals?

#ifndef __ALGORITHMAN__AUTOMATON_H
    #define __ALGORITHMAN__AUTOMATON_H

    #include<vector>
    #include<map>
    #include<set>
    #include<iostream>
    #include<string>
    #include<queue>

    namespace AlgorithMan
    {

        class Transition;
        class DeltaTransition;
        class Automaton;

        class State
        {
            friend class Automaton;
            friend class Transition;
            friend class DeltaTransition;
            protected:
                int x;
                int y;
                std::string Label;

                bool InitialState;
                bool FinalState;

                std::set<Transition*> Transitions;
                std::set<Transition*> IncomingTransitions;

                virtual State* Clone(std::map<const State*, State*>& clones) const;
            public:
                State(int x=0, int y=0, std::string Label="");
                virtual ~State();

                void AddTransition(Transition* transition);

                void SetInitial();
                bool IsInitial();
                void UnsetInitial();

                void SetFinal();
                bool IsFinal();
                void UnsetFinal();
        };




        class Transition
        {
            friend class Automaton;
            friend class State;
            protected:
                State *from;
                State *to;

                Transition(State *from=NULL, State *to=NULL);
            public:
                virtual ~Transition();
                virtual Transition* Clone(std::map<const State*, State*>& clones) const;
                void setFrom(State* from);
                void setTo(State* to);
        };

        typedef Transition EpsilonTransition;

        class DeltaTransition : public Transition
        {
            friend class Automaton;
            friend class State;
            protected:
                char character;

                DeltaTransition(char c, State *from=NULL, State *to=NULL);
                Transition* Clone(std::map<const State*, State*>& clones) const;
            public:
                virtual ~DeltaTransition();
        };




        class Automaton
        {
            protected:
                std::set<State*> Q;
                std::set<char> Sigma;

            public:
                Automaton();
                Automaton(const Automaton& A); // Copy Constructor
                Automaton(Automaton&& A); // Move Constructor
                Automaton* Clone() const;
                virtual ~Automaton();

                void Insert(const Automaton& A);

                void AddState(State* state);
                void RemoveState(State* state);

                bool Deterministic() const;
                bool Accepts(std::istream&) const;
                bool Accepts(const std::string&) const;
                //Automaton Complement() const;
                //Automaton Reverse() const;
                //bool Empty() const;
                //bool Universal() const;
                //bool Finite() const;

                void EpsilonHull(std::set<State*>& states) const;
                /*
                Automaton Determinized() const;
                Automaton EpsilonElimination() const;
                Automaton EpsilonFreeDeterminization() const;
                Automaton Minimized() const;

                bool Isomorphic(const Automaton& A) const;
                bool operator<=(const Automaton& A) const;
                bool operator<(const Automaton& A) const;
                bool operator>=(const Automaton& A) const;
                bool operator>(const Automaton& A) const;
                bool operator==(const Automaton& A) const;
                bool operator!=(const Automaton& A) const;
                */
            protected:
                //Automaton Combine(const Automaton& A, bool AnorB, bool onlyA, bool onlyB, bool AandB, int BfinalId=1) const;

            public:
                /*
                Automaton Intersection(const Automaton& A) const;
                Automaton operator+(const Automaton& A) const;
                Automaton operator-(const Automaton& A) const;
                Automaton operator*(const Automaton& A) const;
                Automaton KleeneHull() const;
                Automaton KleenePlus() const;
                Automaton Quotient(const Automaton& A) const;
                Automaton SynchronizedProduct(const Automaton& A) const; // one automata doesnt have the symbol in its alphabet => it ignores it
                Automaton ShuffleProduct(const Automaton& A) const;

                void Gastex(ostream& os, int h, int w) const;
                string Gastex(int h = 100, int w = 100) const;

                void CCode(ostream& os) const;
                string CCode() const;

                void MyhillNerode(ostream& os) const;
                string MyhillNerode() const;
                */
        };
    }

#endif
